export class HeadsIndex {
    _index = new Map();
    _headsCache;
    _sortFn;
    constructor(properties) {
        this._sortFn = properties.sortFn;
        this.reset(properties.entries);
    }
    reset(entries) {
        this._index.clear();
        (Array.isArray(entries) ? entries : Object.values(entries)).forEach((entry) => {
            if (!entry.hash) {
                throw new Error("Unexpected");
            }
            this._index.set(entry.hash, entry);
        });
        this._headsCache = undefined;
    }
    get(hash) {
        return this._index.get(hash);
    }
    del(hash) {
        const _deleted = this._index.delete(hash);
        this._headsCache = undefined; // TODO do smarter things here, only remove the element needed (?)
    }
    /**
     * Returns an array of heads.
     * @returns {Array<Entry<T>>}
     */
    get heads() {
        if (this._headsCache != undefined) {
            return [...this._headsCache];
        }
        this._headsCache = [...this._index.values()]
            .sort(this._sortFn)
            .reverse();
        return this.heads;
    }
}
//# sourceMappingURL=heads.js.map